import sys
from PySide.QtGui import *
from PySide.QtCore import *
from Thumbnailer import Thumbnails

class ApplicationWindow(QWidget):
    def __init__(self, path=""):
        super(ApplicationWindow, self).__init__()
        self.path = path
        if self.path == "":
            self.path = "/home/user/MyDocs/.images"
        self.widgets = {}
        self.currentRow = 0
        self.currentColumn = 0
        self.colNumber = 5

        self.description = QLabel("This demo presents all signals from Thumbnailer")
        self.directory = QLineEdit(self.path)
        self.action = QPushButton("Start")
        self.stopAction = QPushButton("Cancel")
        self.vbox = QVBoxLayout()
        self.hbox = QHBoxLayout()
        self.thumbs = QGridLayout()
        self.progress = QProgressBar()
        self.thumb = None
        self.progress.setMinimum(0)
        self.progress.setMaximum(100)
        self.progress.setValue(0)

        self.sendRemaining = QCheckBox("Send remainig signals (dequeued)?")
        self.sendRemaining.setChecked(True)

        self.setLayout(self.vbox)
        self.vbox.addWidget(self.description)
        self.vbox.addWidget(self.sendRemaining)

        self.hbox.addWidget(self.directory)
        self.hbox.addWidget(self.action)
        self.hbox.addWidget(self.stopAction)

        self.vbox.addLayout(self.hbox)

        self.scrollArea = QScrollArea()
        self.widget = QWidget()
        self.widget.setLayout(self.thumbs)
        self.widget.show()
        self.scrollArea.setWidget(self.widget)
        self.scrollArea.setWidgetResizable(True)

        self.vbox.addWidget(self.scrollArea)
        self.vbox.addWidget(self.progress)

        self.action.clicked.connect(self.loadThumbnails)
        self.stopAction.clicked.connect(self.cancelRequest)

    def cancelRequest(self):
        if self.thumb == None:
            return

        send = self.sendRemaining.isChecked()
        self.thumb.cancel(send)

    def loadThumbnails(self):
        self.progress.setValue(0)

        dir = QDir(self.path)
        filters = ["*.jpg", "*.jpeg", "*.JPG", "*.JPEG", "*.avi"]
        dir.setNameFilters(filters)
        files = dir.entryList(QDir.Files | QDir.Readable)

        uris = []
        mimes = []

        for file in files:
            file = "file://" + self.path + "/" + file
            uris.append(QUrl(file))
            if file.endswith(".avi"):
                mimes.append("video/x-msvideo")
            else:
                mimes.append("image/jpeg")
        self.progress.setMaximum(len(uris))

        if self.thumb == None:
            self.thumb = Thumbnails.Thumbnailer(QUrl(""))
            self.thumb.started.connect(self.loadingStarted)
            self.thumb.finished.connect(self.loadingFinished)
            QObject.connect(self.thumb, SIGNAL('thumbnail(QUrl,QUrl,QPixmap,QString)'), self.thumbnail)

        done = False
        done = self.thumb.request(uris, mimes, True) # true - send also a pixmap with the thumbnail
        if done:
            print "All thumbnails were ready at the request time - no need for dbus traffic!"

    def getImageArea(self, uri):
        if not uri in self.widgets:
            label = QLabel()
            label.setVisible(True)
            self.widgets[uri.toString()] = label
            self.thumbs.addWidget(label, self.currentRow, self.currentColumn)
            self.currentColumn += 1

            if self.currentColumn >= self.colNumber:
                self.currentColumn = 0
                self.currentRow += 1
        return self.widgets[uri.toString()]
            
    def loadingStarted(self):
        self.description.setText("Requesting and proccessing thumbnails...")

    def loadingFinished(self, left):
        print "There are other " + str(left) + " request(s) left."
        self.description.setText("Request finished")
        self.progress.setValue(self.progress.maximum())

    def thumbnail(self, uri, thumb, pixmap, foo):
        self.getImageArea(uri).setPixmap(pixmap)
        self.progress.setValue(self.progress.value() + 1)

if __name__ == '__main__':
    application = QApplication(sys.argv)
    if len(sys.argv) > 1:
        mw = ApplicationWindow(sys.argv[1])
    else:
        mw = ApplicationWindow()

    mw.showMaximized()

    sys.exit(application.exec_())

